@@ -1,15 +1,16 @@
+#include "config.h"
 #include "pot.h"
 #include <locale.h>
 
 /*************************************************************************
  * if you want to turn off gettext without changing sources edit pot.h 
  *************************************************************************/
- 
+
 void gettexton(void)
 {
-#ifdef __GETTEXT__
- setlocale(LC_ALL, "");
- bindtextdomain("quota","/usr/share/locale");
- textdomain("quota");
+#ifdef USE_GETTEXT
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
+	textdomain(PACKAGE);
 #endif
-}
\ No newline at end of file
+}
