@@ -538,6 +538,7 @@ static int scan_dir(const char *pathname)
 		if ((lstat(de->d_name, &st)) == -1) {
 			errstr(_("lstat: Cannot stat `%s/%s': %s\nGuess you'd better run fsck first !\nexiting...\n"),
 				pathname, de->d_name, strerror(errno));
+			closedir(dp);
 			goto out;
 		}
 
@@ -924,6 +925,7 @@ static int check_dir(struct mount_entry *mnt)
 	struct stat st;
 	int remounted = 0;
 	int failed = 0;
+	int ret;
 
 	if (lstat(mnt->me_dir, &st) < 0)
 		die(2, _("Cannot stat mountpoint %s: %s\n"), mnt->me_dir, strerror(errno));
@@ -939,14 +941,22 @@ static int check_dir(struct mount_entry *mnt)
 	 */
 	if (cfmt == QF_XFS)
 		goto start_scan;
-	if (ucheck)
-		if (process_file(mnt, USRQUOTA) < 0)
+	if (ucheck) {
+		ret = process_file(mnt, USRQUOTA);
+		if (ret < 0) {
+			failed |= ret;
 			ucheck = 0;
-	if (gcheck)
-		if (process_file(mnt, GRPQUOTA) < 0)
+		}
+	}
+	if (gcheck) {
+		ret = process_file(mnt, GRPQUOTA);
+		if (ret < 0) {
+			failed |= ret;
 			gcheck = 0;
+		}
+	}
 	if (!ucheck && !gcheck)	/* Nothing to check? */
-		return 0;
+		return failed;
 	if (!(flags & FL_NOREMOUNT)) {
 		/* Now we try to remount fs read-only to prevent races when scanning filesystem */
 		if (mount
@@ -978,8 +988,11 @@ start_scan:
 	    !strcmp(mnt->me_type, MNTTYPE_EXT3) ||
 	    !strcmp(mnt->me_type, MNTTYPE_NEXT3) ||
 	    !strcmp(mnt->me_type, MNTTYPE_EXT4)) {
-		if ((failed = ext2_direct_scan(mnt->me_devname)) < 0)
+		ret = ext2_direct_scan(mnt->me_devname);
+		if (ret < 0) {
+			failed |= ret;
 			goto out;
+		}
 	}
 	else {
 #else
@@ -987,8 +1000,11 @@ start_scan:
 #endif
 		if (flags & FL_VERYVERBOSE)
 			putchar('\n');
-		if ((failed = scan_dir(mnt->me_dir)) < 0)
+		ret = scan_dir(mnt->me_dir);
+		if (ret < 0) {
+			failed |= ret;
 			goto out;
+		}
 	}
 	dirs_done++;
 	if (flags & FL_VERBOSE || flags & FL_DEBUG)
@@ -1027,7 +1043,8 @@ static int detect_filename_format(struct mount_entry *mnt, int type)
 	int fmt;
 
 	if (strcmp(mnt->me_type, MNTTYPE_XFS) == 0 ||
-	    strcmp(mnt->me_type, MNTTYPE_GFS2) == 0)
+	    strcmp(mnt->me_type, MNTTYPE_GFS2) == 0 ||
+	    strcmp(mnt->me_type, MNTTYPE_EXFS) == 0)
 		return QF_XFS;
 
 	if (type == USRQUOTA) {
@@ -1101,7 +1118,7 @@ static int compatible_fs_qfmt(char *fstype, int fmt)
 {
 	/* We never check XFS, NFS, and filesystems supporting VFS metaformat */
 	if (!strcmp(fstype, MNTTYPE_XFS) || nfs_fstype(fstype) ||
-	    meta_qf_fstype(fstype))
+	    meta_qf_fstype(fstype) || !strcmp(fstype, MNTTYPE_EXFS))
 		return 0;
 	/* In all other cases we can pick a format... */
 	if (fmt == -1)
