@@ -31,13 +31,13 @@
 
 static int get_service_port(u_long number, const char *proto)
 {
-	char rpcdata [1024], servdata [1024];
-	struct rpcent rpcbuf, *rpcp = NULL;
+	char servdata [1024];
+	struct rpcent *rpcp = NULL;
 	struct servent servbuf, *servp = NULL;
 	int ret;
 
-	ret = getrpcbynumber_r(number, &rpcbuf, rpcdata, sizeof(rpcdata), &rpcp);
-	if (ret == 0 && rpcp != NULL) {
+	rpcp = getrpcbynumber(number);
+	if (rpcp != NULL) {
 		/* First try name */
 		ret = getservbyname_r(rpcp->r_name, proto, &servbuf, servdata,
 		                       sizeof servdata, &servp);
@@ -118,6 +118,15 @@ static int svc_create_sock(struct addrinfo *ai)
 		return -1;
 	}
 
+	if (ai->ai_family == AF_INET6) {
+		if (setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY,
+				&optval, sizeof(optval)) < 0) {
+			errstr(_("Cannot set IPv6 socket options: %s\n"), strerror(errno));
+			close(fd);
+			return -1;
+		}
+	}
+
 	if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval)) < 0) {
 		errstr(_("Cannot set socket options: %s\n"), strerror(errno));
 		close(fd);
@@ -129,6 +138,15 @@ static int svc_create_sock(struct addrinfo *ai)
 		close(fd);
 		return -1;
 	}
+
+	if (ai->ai_protocol == IPPROTO_TCP) {
+		if (listen(fd, SOMAXCONN) < 0) {
+			errstr(_("Cannot listen to address: %s\n"), strerror(errno));
+			close(fd);
+			return -1;
+		}
+	}
+
 	return fd;
 }
 
