@@ -30,7 +30,9 @@
 #include <time.h>
 #include <getopt.h>
 #include <locale.h>
+#ifdef HAVE_NL_LANGINFO
 #include <langinfo.h>
+#endif
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/utsname.h>
@@ -84,7 +86,6 @@
 #define FL_USER 1
 #define FL_GROUP 2
 #define FL_NOAUTOFS 4
-#define FL_SHORTNUMS 8
 #define FL_NODETAILS 16
 
 struct usage {
@@ -109,13 +110,13 @@ struct configparams {
 	char *user_signature;
 	char *group_message;
 	char *group_signature;
+	time_t cc_before;
+#ifdef USE_LDAP_MAIL_LOOKUP
 	int use_ldap_mail; /* 0 */
+	int ldap_is_setup; /* 0 */
 	int ldap_starttls; /* 0 */
 	int ldap_tls; /* LDAP_OPT_X_TLS_NEVER */
 	int ldap_vers; /* LDAP_VERSION3 */
-	time_t cc_before;
-#ifdef USE_LDAP_MAIL_LOOKUP
-	int ldap_is_setup; /* 0 */
 	char ldap_host[CNF_BUFFER];
 	int ldap_port;
 	char ldap_uri[CNF_BUFFER];
@@ -155,6 +156,7 @@ static char *hostname, *domainname;
 static quotatable_t *quotatable;
 static int adminscnt, adminsalloc;
 static struct adminstable *adminstable;
+static enum s2s_unit spaceunit = S2S_NONE, inodeunit = S2S_NONE;
 
 /*
  * Global pointers to list.
@@ -410,7 +412,13 @@ static char *lookup_user(struct configparams *config, char *user)
 	}
 
 	/* search for the offender_name in ldap */
-	snprintf(searchbuf, 256, "(%s=%s)", config->ldap_search_attr, user);
+	if (256 <= snprintf(searchbuf, 256, "(%s=%s)", config->ldap_search_attr,
+		    user)) {
+		errstr(_("Could not format LDAP search filter for %s user and "
+			"%s search attribute due to excessive length.\n"),
+			user, config->ldap_search_attr);
+		return NULL;
+	}
 	ret = ldap_search_ext_s(ldapconn,
 		config->ldap_basedn, LDAP_SCOPE_SUBTREE,
 		searchbuf, NULL, 0, NULL, NULL, NULL,
@@ -502,7 +510,7 @@ static int mail_user(struct offenderlist *offender, struct configparams *config)
 		char *cc_to = lookup_user(config, config->cc_to);
 
 		if (cc_to) {
-			fprintf(fp, "Cc: %s\n", config->cc_to);
+			fprintf(fp, "Cc: %s\n", cc_to);
 			free(cc_to);
 		}
 	}
@@ -546,9 +554,9 @@ static int mail_user(struct offenderlist *offender, struct configparams *config)
 				difftime2str(dqb->dqb_btime, timebuf);
 			else
 				timebuf[0] = '\0';
-			space2str(toqb(dqb->dqb_curspace), numbuf[0], flags & FL_SHORTNUMS);
-			space2str(dqb->dqb_bsoftlimit, numbuf[1], flags & FL_SHORTNUMS);
-			space2str(dqb->dqb_bhardlimit, numbuf[2], flags & FL_SHORTNUMS);
+			space2str(toqb(dqb->dqb_curspace), numbuf[0], spaceunit);
+			space2str(dqb->dqb_bsoftlimit, numbuf[1], spaceunit);
+			space2str(dqb->dqb_bhardlimit, numbuf[2], spaceunit);
 			fprintf(fp, "%c%c %7s %7s %7s %6s",
 			        dqb->dqb_bsoftlimit && toqb(dqb->dqb_curspace) >= dqb->dqb_bsoftlimit ? '+' : '-',
 				dqb->dqb_isoftlimit && dqb->dqb_curinodes >= dqb->dqb_isoftlimit ? '+' : '-',
@@ -557,9 +565,9 @@ static int mail_user(struct offenderlist *offender, struct configparams *config)
 				difftime2str(dqb->dqb_itime, timebuf);
 			else
 				timebuf[0] = '\0';
-			number2str(dqb->dqb_curinodes, numbuf[0], flags & FL_SHORTNUMS);
-			number2str(dqb->dqb_isoftlimit, numbuf[1], flags & FL_SHORTNUMS);
-			number2str(dqb->dqb_ihardlimit, numbuf[2], flags & FL_SHORTNUMS);
+			number2str(dqb->dqb_curinodes, numbuf[0], inodeunit);
+			number2str(dqb->dqb_isoftlimit, numbuf[1], inodeunit);
+			number2str(dqb->dqb_ihardlimit, numbuf[2], inodeunit);
 			fprintf(fp, " %7s %5s %5s %6s\n\n", numbuf[0], numbuf[1], numbuf[2], timebuf);
 		}
 	}
@@ -723,19 +731,21 @@ static int readconfigfile(const char *filename, struct configparams *config)
 	(config->charset)[0] = '\0';
 	setlocale(LC_ALL, NULL);
 	locale = setlocale(LC_MESSAGES, NULL);
+#ifdef HAVE_NL_LANGINFO
 	if (locale && strcasecmp(locale, "posix") && strcasecmp(locale, "c")) {
 		locale = nl_langinfo(CODESET);
 		sstrncpy(config->charset, locale, CNF_BUFFER);
 	}
+#endif
 	maildev[0] = 0;
 	config->user_signature = config->user_message = config->group_signature = config->group_message = NULL;
+	config->cc_before = -1;
+
+#ifdef USE_LDAP_MAIL_LOOKUP
 	config->use_ldap_mail = 0;
 	config->ldap_starttls = 0;
 	config->ldap_tls = LDAP_OPT_X_TLS_NEVER;
 	config->ldap_vers = LDAP_VERSION3;
-	config->cc_before = -1;
-
-#ifdef USE_LDAP_MAIL_LOOKUP
 	config->ldap_port = config->ldap_is_setup = 0;
 	config->ldap_host[0] = 0;
 	config->ldap_uri[0] = 0;
@@ -820,6 +830,18 @@ static int readconfigfile(const char *filename, struct configparams *config)
 				create_eoln(config->group_signature);
 				verify_format(config->group_signature, "GROUP_SIGNATURE");
 			}
+			else if (!strcmp(var, "CC_BEFORE")) {
+				int num;
+				char unit[10];
+
+				if (sscanf(value, "%d%s", &num, unit) != 2)
+					goto cc_parse_err;
+				if (str2timeunits(num, unit, &config->cc_before) < 0) {
+cc_parse_err:
+					die(1, _("Cannot parse time at CC_BEFORE variable (line %d).\n"), line);
+				}
+			}
+#ifdef USE_LDAP_MAIL_LOOKUP
 			else if (!strcmp(var, "LDAP_MAIL")) {
 				if(strcasecmp(value, "true") == 0) 
 					config->use_ldap_mail = 1;
@@ -846,18 +868,6 @@ static int readconfigfile(const char *filename, struct configparams *config)
 				else
 					config->ldap_starttls = 0;
 			}
-			else if (!strcmp(var, "CC_BEFORE")) {
-				int num;
-				char unit[10];
-
-				if (sscanf(value, "%d%s", &num, unit) != 2)
-					goto cc_parse_err;
-				if (str2timeunits(num, unit, &config->cc_before) < 0) {
-cc_parse_err:
-					die(1, _("Cannot parse time at CC_BEFORE variable (line %d).\n"), line);
-				}
-			}
-#ifdef USE_LDAP_MAIL_LOOKUP
 			else if (!strcmp(var, "LDAP_HOST"))
 				sstrncpy(config->ldap_host, value, CNF_BUFFER);
 			else if (!strcmp(var, "LDAP_PORT"))
@@ -889,7 +899,14 @@ cc_parse_err:
 	if (config->use_ldap_mail)
 	{
 		if (!config->ldap_uri[0]) {
-			snprintf(config->ldap_uri, CNF_BUFFER, "ldap://%s:%d", config->ldap_host, config->ldap_port);
+			if (CNF_BUFFER <= snprintf(config->ldap_uri, CNF_BUFFER,
+				    "ldap://%s:%d", config->ldap_host,
+				    config->ldap_port)) {
+				errstr(_("Could not format LDAP URI because "
+					    "it's longer than %d bytes.\n"),
+					    CNF_BUFFER);
+				return -1;
+			}
 			errstr(_("LDAP library version >= 2.3 detected. Please use LDAP_URI instead of hostname and port.\nGenerated URI %s\n"), config->ldap_uri);
 		}
 	}
@@ -1012,7 +1029,12 @@ static void usage(void)
 	errstr(_("Usage:\n  warnquota [-ugsid] [-F quotaformat] [-c configfile] [-q quotatabfile] [-a adminsfile] [filesystem...]\n\n\
 -u, --user                      warn users\n\
 -g, --group                     warn groups\n\
--s, --human-readable            send information in more human friendly units\n\
+-s, --human-readable[=units]    display numbers in human friendly units (MB,\n\
+                                GB, ...). Units can be also specified\n\
+                                explicitely by an optional argument in format\n\
+                                [kgt],[kgt] where the first character specifies\n\
+                                space units and the second character specifies\n\
+                                inode units\n\
 -i, --no-autofs                 avoid autofs mountpoints\n\
 -d, --no-details                do not send quota information itself\n\
 -F, --format=formatname         use quotafiles of specific format\n\
@@ -1038,12 +1060,12 @@ static void parse_options(int argcnt, char **argstr)
 		{ "quota-tab", 1, NULL, 'q' },
 		{ "admins-file", 1, NULL, 'a' },
 		{ "no-autofs", 0, NULL, 'i' },
-		{ "human-readable", 0, NULL, 's' },
+		{ "human-readable", 2, NULL, 's' },
 		{ "no-details", 0, NULL, 'd' },
 		{ NULL, 0, NULL, 0 }
 	};
  
-	while ((ret = getopt_long(argcnt, argstr, "ugVF:hc:q:a:isd", long_opts, NULL)) != -1) {
+	while ((ret = getopt_long(argcnt, argstr, "ugVF:hc:q:a:is::d", long_opts, NULL)) != -1) {
 		switch (ret) {
 		  case '?':
 		  case 'h':
@@ -1074,7 +1096,11 @@ static void parse_options(int argcnt, char **argstr)
 			flags |= FL_NOAUTOFS;
 			break;
 		  case 's':
-			flags |= FL_SHORTNUMS;
+			inodeunit = spaceunit = S2S_AUTO;
+			if (optarg) {
+				if (unitopt2unit(optarg, &spaceunit, &inodeunit) < 0)
+					die(1, _("Bad output format units for human readable output: %s\n"), optarg);
+			}
 			break;
 		  case 'd':
 			flags |= FL_NODETAILS;
